#ifndef AMI_SMP_H_
#define AMI_SMP_H_

#include <dk_buttons_and_leds.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Mimimal number of ZCBOR encoder states to provide full encoder functionality. */
#define CBOR_ENCODER_STATE_NUM 2

/* Number of ZCBOR decoder states required to provide full decoder functionality plus one
 * more level for decoding nested map received in response to SMP echo command.
 */
#define CBOR_DECODER_STATE_NUM 3

#define CBOR_MAP_MAX_ELEMENT_CNT 2
#define CBOR_BUFFER_SIZE 128

#define SMP_ECHO_MAP_KEY_MAX_LEN 2
#define SMP_ECHO_MAP_VALUE_MAX_LEN 30

#define AMI_SMP_WORK_UPLOAD                      0
#define AMI_SMP_WORK_LIST                        1
#define AMI_SMP_WORK_TEST                        2
#define AMI_SMP_WORK_RESET                       3
#define AMI_SMP_WORK_CONFIRM                     4


#define AMI_SMP_DFU_ERR_OK                       0
#define AMI_SMP_DFU_ERR_RING_UNDERRUN            1
#define AMI_SMP_DFU_ERR_UPLOAD_ENCODE            2
#define AMI_SMP_DFU_ERR_CONFIRM_ENCODE           3
#define AMI_SMP_DFU_ERR_TEST_ENCODE              4
#define AMI_SMP_DFU_ERR_SEND                     5
#define AMI_SMP_DFU_ERR_LIST_BAD_OPER            6
#define AMI_SMP_DFU_ERR_LIST_BAD_GROUP           7
#define AMI_SMP_DFU_ERR_LIST_BAD_CMD             8
#define AMI_SMP_DFU_ERR_LIST_START_DECODE        9
#define AMI_SMP_DFU_ERR_LIST_IMAGE_DECODE        10
#define AMI_SMP_DFU_ERR_LIST_IMAGE_LIST          11
#define AMI_SMP_DFU_ERR_LIST_SLOT_KEY            12
#define AMI_SMP_DFU_ERR_LIST_SLOT_VAL            13
#define AMI_SMP_DFU_ERR_LIST_DECODE_VER          14
#define AMI_SMP_DFU_ERR_LIST_VER_VAL             15
#define AMI_SMP_DFU_ERR_LIST_DECODE_HASH         16
#define AMI_SMP_DFU_ERR_LIST_HASH_VAL            17
#define AMI_SMP_DFU_ERR_LIST_DECODE_BOOT         18
#define AMI_SMP_DFU_ERR_LIST_BOOT_VAL            19
#define AMI_SMP_DFU_ERR_LIST_DECODE_PEND         20
#define AMI_SMP_DFU_ERR_LIST_PEND_VAL            21
#define AMI_SMP_DFU_ERR_LIST_DECODE_CONFIRM      22
#define AMI_SMP_DFU_ERR_LIST_CONFIRM_VAL         23
#define AMI_SMP_DFU_ERR_LIST_DECODE_ACT          24
#define AMI_SMP_DFU_ERR_LIST_ACT_VAL             25
#define AMI_SMP_DFU_ERR_LIST_DECODE_PERM         26
#define AMI_SMP_DFU_ERR_LIST_PERM_VAL            27
#define AMI_SMP_DFU_ERR_LIST_SEND                28
#define AMI_SMP_DFU_ERR_TEST_SEND                40

#define AMI_SMP_DFU_ERR_LOOP_TIMEOUT             99
#define AMI_SMP_DFU_ERR_UPLOAD                   100

#define AMI_SMP_DFU_MAX_LOOP_WAIT_MS             30000    // 30 seconds

extern void ami_smp_init(void);
extern int  ami_smp_submit_work(uint8_t which);
extern int  ami_smp_assign_handles(struct bt_gatt_dm *dm);

#ifdef __cplusplus
}
#endif

/**
 *@}
 */

#endif /* AMI_SMP_H_ */
